<?php
/**
 * @package     Joomla.Administrator
 * @subpackage  com_joomlaextphp7checker
 *
 * @copyright   Copyright (C) 2016 Strato AG All rights reserved.
 * @license     GNU General Public License version 3; see LICENSE.txt
 */
defined('_JEXEC') or die('Restricted access');

/**
 * Class Php7ExtCheckerModelCheck
 *
 * @since  0.0.1
 */
class Php7ExtCheckerModelCheck extends JModelList
{
	/**
	 * Retrieves the data from the Form and checks the installed extensions. Gives the result back to the view for display
	 *
	 * @return array
	 */
	public function getData()
	{
		$sheetUrl = "https://docs.google.com/spreadsheets/d/1HJj02l3ByBvJF-Q67XXuGmrFvS8iIBEnDEcVcmwqzLc/pub?gid=0&single=true&output=csv";
		$document = [];
		$sheetHandle = fopen($sheetUrl, 'r');

		while (($line = fgetcsv($sheetHandle, 1000, ',')) !== false)
		{
			$document[] = $line;
		}

		fclose($sheetHandle);

		$extList = [];

		for ($lineIndex = 1; $lineIndex < count($document); $lineIndex++)
		{
			$key = $document[$lineIndex][2] != '' ? $document[$lineIndex][2] : $document[$lineIndex][1];
			$extList[$key] = (object) [
				"name" => $document[$lineIndex][1],
				"version" => $document[$lineIndex][3],
				"isJ35Compatible" => $document[$lineIndex][4] === 'Yes' ? true : false,
				"isPhp7Compatible" => $document[$lineIndex][5] === 'Yes' ? true : false,
			];
		}

		$installedExts = $this->getInstalledExtensions();
		$this->determineExtensionState($extList, $installedExts);
		usort($installedExts, array('Php7ExtCheckerModelCheck','sortByDisplayRank'));

		return $installedExts;
	}

	/**
	 * retrieves all installed Extensions from the database
	 *
	 * @return array
	 */
	private function getInstalledExtensions()
	{
		$lang = JFactory::getLanguage();
		$db = JFactory::getDbo();
		$componentsFolder = JPATH_ADMINISTRATOR . '/components/';

		$query = $db->getQuery(true);
		$query	->select($db->quoteName(array('name', 'manifest_cache', 'type')))
				->from($db->quoteName('#__extensions'));
		$db->setQuery($query);
		$results = $db->loadObjectList();
		$extensions = [];

		foreach ($results as $extension)
		{
			$source = $componentsFolder . $extension->name;
			$lang->load("{$extension->name}.sys", JPATH_ADMINISTRATOR, null, false, false)
				||    $lang->load("{$extension->name}.sys", $source, null, false, false)
				||    $lang->load("{$extension->name}.sys", JPATH_ADMINISTRATOR, $lang->getDefault(), false, false)
				||    $lang->load("{$extension->name}.sys", $source, $lang->getDefault(), false, false);

			$decode = json_decode($extension->manifest_cache);

			if ((isset($decode->authorEmail))
				&& $decode->authorEmail != 'admin@joomla.org'
				&& ($extension->type == 'component' || $extension->type == 'plugin' || $extension->type == 'module')
				&& $extension->name != 'com_php7extchecker')
			{
				$extensions[] = (object) [
					"name" => strtolower($extension->name),
					"caption" => JText::sprintf($extension->name),
					"version" => $decode->version,
					"author" => $decode->author,
					"authorEmail" => $decode->authorEmail,
					"type" => $extension->type,
				];
			}
		}

		return $extensions;
	}

	/**
	 * checks the installed extensions against the List retrieved prior
	 *
	 * @param   array  $extensionList  List coming from the server of all extensions known to be PHP7-Compatible
	 * @param   array  $installedExts  List of all installed Extensions
	 *
	 * @return void
	 */
	private function determineExtensionState($extensionList, $installedExts)
	{

		foreach ($installedExts as $ext)
		{
			if ((isset($extensionList[$ext->name]))
				&& version_compare($ext->version, $extensionList[$ext->name]->version, '>='))
			{
				$ext->isPhp7Compatible = $extensionList[$ext->name]->isPhp7Compatible;
				$ext->mustUpgrade = false;
				$ext->displayRank = $ext->isPhp7Compatible ? 1 : 3;
				$ext->displayClass = $ext->isPhp7Compatible ? 'compatible' : 'uncompatible';
				$ext->label = $ext->isPhp7Compatible ? JText::_('COM_PHP7EXTCHECKER_COMPATIBLE') : JText::_('COM_PHP7EXTCHECKER_NOT_COMPATIBLE');
			} elseif (isset($extensionList[$ext->name]) && $extensionList[$ext->name]->isPhp7Compatible) {
				$ext->isPhp7Compatible = true;
				$ext->mustUpgrade = true;
				$ext->displayRank = 2;
				$ext->displayClass = 'must-upgrade';
				$ext->label = JText::_('COM_PHP7EXTCHECKER_MUST_UPGRADE');
			} elseif (isset($extensionList[$ext->name])) {
				$ext->isPhp7Compatible = false;
				$ext->mustUpgrade = true;
				$ext->displayRank = 4;
				$ext->displayClass = 'outdated';
				$ext->label = JText::_('COM_PHP7EXTCHECKER_NOT_COMPATIBLE_OUTDATED');
			} else {
				$ext->isPhp7Compatible = null;
				$ext->mustUpgrade = null;
				$ext->displayRank = 5;
				$ext->displayClass = 'no-infos';
				$ext->label = JText::_('COM_PHP7EXTCHECKER_NO_INFOS');
			}
		}
	}

	/**
	 * private function for uSort: sorts all extensions according to their display-rank
	 *
	 * @param   object  $ext1  Extension 1
	 * @param   object  $ext2  Extension 2
	 *
	 * @return  int
	 */
	private static function sortByDisplayRank($ext1, $ext2)
	{
		if ($ext1->displayRank == $ext2->displayRank)
		{
			return 0;
		}

		return $ext1->displayRank < $ext2->displayRank ? -1 : 1;
	}
}